"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.connect = void 0;
var logFunctions = {
    log: console.log,
    info: console.info,
    error: console.error,
    debug: console.debug,
    trace: console.trace
};
var reconnect = null;
var STATUS_NOT_CONNECTED = 'STATUS_NOT_CONNECTED';
var STATUS_CONNECTING = 'STATUS_CONNECTING';
var STATUS_CONNECTED = 'STATUS_CONNECTED';
var status = STATUS_NOT_CONNECTED;
function connect(url) {
    if (url === void 0) { url = 'ws://localhost:12345'; }
    if (status === STATUS_CONNECTING)
        return;
    status = STATUS_CONNECTING;
    logFunctions.info("LOCAL: Trying to connect to remote log server " + url.toString());
    var connection = new WebSocket(url);
    connection.onopen = onOpen;
    connection.onclose = onClose;
}
exports.connect = connect;
function onOpen(event) {
    status = STATUS_CONNECTED;
    logFunctions.info('LOCAL: Connection to remote log server opened');
    if (reconnect)
        clearInterval(reconnect);
    registerHandlers(this);
}
function onClose(error) {
    status = STATUS_NOT_CONNECTED;
    logFunctions.info('LOCAL: Connection to remote log server closed');
    unregisterHandlers();
    reconnect = setInterval(function () { return connect(); }, 2500);
}
function logger(connection, callback, type, payload) {
    callback.apply(void 0, payload);
    connection.send(JSON.stringify({ type: type, payload: payload }));
}
function registerHandlers(connection) {
    console.log = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return logger(connection, logFunctions.log, 'log', args);
    };
    console.info = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return logger(connection, logFunctions.info, 'info', args);
    };
    console.error = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return logger(connection, logFunctions.error, 'error', args);
    };
    console.debug = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return logger(connection, logFunctions.debug, 'debug', args);
    };
    console.trace = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return logger(connection, logFunctions.trace, 'trace', args);
    };
}
function unregisterHandlers() {
    console.log = logFunctions.log;
    console.info = logFunctions.info;
    console.error = logFunctions.error;
    console.debug = logFunctions.debug;
    console.trace = logFunctions.trace;
}
//# sourceMappingURL=client.js.map